@@ -2,7 +2,8 @@ import requests
 from os.path import join, exists, dirname
 import os
 import glob
-from jaaql.constants import DIR__config, FILE__config, CONFIG_KEY__server, CONFIG_KEY_SERVER__port, ENVIRON__install_path
+from jaaql.constants import DIR__config, FILE__config, CONFIG_KEY__server, CONFIG_KEY_SERVER__port, ENVIRON__install_path, PORT__ems
+from jaaql.config_constants import *
 from jaaql.db.db_interface import DBInterface
 from jaaql.db.db_utils import create_interface
 from jaaql.constants import VAULT_KEY__jaaql_lookup_connection
@@ -64,6 +65,20 @@ def await_jaaql_installation(config, is_gunicorn: bool):
         time.sleep(5)
 
 
+def get_external_url(config):
+    return config[CONFIG_KEY__swagger][CONFIG_KEY_SWAGGER__url]
+
+
+def await_ems_startup():
+    while True:
+        try:
+            if requests.get("http://127.0.0.1:" + str(PORT__ems) + "/").status_code == 200:
+                break
+        except:
+            pass
+        time.sleep(5)
+
+
 def get_jaaql_connection(config, vault):
     jaaql_uri = vault.get_obj(VAULT_KEY__jaaql_lookup_connection)
     address, port, db, username, password = DBInterface.fracture_uri(jaaql_uri)
